;Project Lynxmotion Phoenix
;Description: Phoenix, driver file
; 			  SSC Driver - Contains all of the code that knows anything about the SSC-32
;					 	   It also has timer functions as this may also change with drivers...
;						   Can be used with V2.1 and above
;
;SSC Driver version: V1.0
;Date: 19-04-2011
;Programmer: Jeroen Janssen (aka Xan)
;			 Kurt (aka KurtE)

;====================================================================
lTimerCnt			var long	

;--------------------------------------------------------------------
;[TIMER INTERRUPT INIT]
TIMERINT	con	TIMERAINT
ONASMINTERRUPT TIMERAINT, HANDLE_TIMERA_ASM 

;--------------------------------------------------------------------
;[InitServoDriver] - Initializes the servo driver including the main timer used in the phoenix code
InitTimer:
;Timer
; Timer A init, used for timing of messages and some times for timing code...
  TIMERINT	con	TIMERAINT

  WTIMERTICSPERMSMUL con 64	; BAP28 is 16mhz need a multiplyer and divider to make the conversion with /8192
  WTIMERTICSPERMSDIV con 125  ; 
  TMA = 0	; clock / 8192					; Low resolution clock - used for timeouts...
  ENABLE TIMERINT
return

;==============================================================================
;[Handle_Timer_asm] - Handle timer A overflow in assembly language.  Currently only
;used for timings for debuging the speed of the code
;Now used to time how long since we received a message from the remote.
;this is important when we are in the NEW message mode, as we could be hung
;out with the robot walking and no new commands coming in.
;==============================================================================
BEGINASMSUB 
HANDLE_TIMERA_ASM 
	push.l 	er1                  ; first save away ER1 as we will mess with it. 
	bclr 	#6,@IRR1:8               ; clear the cooresponding bit in the interrupt pending mask 
	mov.l 	@LTIMERCNT:16,er1      ; Add 256 to our counter 
	add.l	#256,er1 
	mov.l 	er1, @LTIMERCNT:16 
	pop.l 	er1 
	rte 
ENDASMSUB 

return		; Put a basic statement before...
;==============================================================================
;[GetCurrentTime] - Gets the Timer value from our overflow counter as well as the TCA counter.  It
;                makes sure of consistancy. That is it is very posible that 
;                after we grabed the timers value it overflows, before we grab the other part
;                so we check to make sure it is correct and if necesary regrab things.
;==============================================================================
GetCurrentTime:
  lCurrentTime = lTimerCnt + TCA
	
  ; handle wrap
  if lTimerCnt <> (lCurrentTime & 0xffffff00) then
	lCurrentTime = lTimerCnt + TCA
  endif

return lCurrentTime
;-------------------------------------------------------------------------------------
;[ConvertTimeMS]
_ttconv	var	long
ConvertTimeMS[_ttconv]:
	return (_ttconv * WTIMERTICSPERMSMUL)/WTIMERTICSPERMSDIV 


;--------------------------------------------------------------------
;[CheckGPEnable] Checks to see if the SSC-32 support the general purpose sequences
CheckGPEnable:
	
  pause 10
  GPEnable=0
  serout cSSC_OUT, cSSC_BAUD, ["ver", 13]
	
GetSSCVersion:
  serin cSSC_IN, cSSC_BAUD, 10000, timeout, [GPVerData(Index)]
  Index = (Index+1)//3 ; shift last 3 chars in data
  goto GetSSCVersion
	
timeout:
  if (GPVerData(0) + GPVerData(1) + GPVerData(2)) = 164 then ; Check if the last 3 chars are G(71) P(80) cr(13)
	return 1
  else
  	sound cSpeakerPin, [40\5000,40\5000]
  endif
return 0

;--------------------------------------------------------------------
;[GP PLAYER]
GPStatSeq       var byte
GPStatFromStep  var byte
GPStatToStep   	var byte
GPStatTime      var byte
GPSMPrev		var sword
GPSeqStart		var	word
GPCntSteps		var byte

GPPlayer:

  ;Start sequence
  IF (GPStart=1) THEN
    disable TIMERINT 		;disable timer interrupt
	gosub GPSeqNumSteps		; gets the count of steps for the sequence.  Should check error state.
    serout cSSC_OUT, cSSC_BAUD, ["PL0SQ", dec GPSeq,"SM", sdec GPSM, 13] ;Start sequence
    GPSMPrev = GPSM

    enable TIMERINT
    GPStart=2   ; set the state to do query...
  ELSEIF (GPStart=2)
    ;See if  GPPlayer has completed sequence
    disable TIMERINT 		;disable timer interrupt
    serout cSSC_OUT, cSSC_BAUD, ["QPL0", 13]
    serin cSSC_IN, cSSC_BAUD, [GPStatSeq, GPStatFromStep, GPStatToStep, GPStatTime]
    ;The ONCE command does not work after PL0SM cmd, therefore our own function for stopping the player
    ;NB! Reuse of the GPVerData, now it contains total # of steps in current sequence
    IF (GPStatFromStep = (GPCntSteps-1))&(GPStatTime=0)THEN;Stop SQ at the last step in SQ, 
      serout cSSC_OUT, cSSC_BAUD, ["PL0", 13] ;Stop player
      GPStart=0
    ELSEIF GPSMPrev <> GPSM
      serout cSSC_OUT, cSSC_BAUD, ["PL0SM", sdec GPSM, 13]   
      GPSMPrev = GPSM
    ENDIF
    enable TIMERINT
  ELSEIF GPStart=0xff
    ; user requested us to abort the sequence.
    disable TIMERINT 		;disable timer interrupt
    serout cSSC_OUT, cSSC_BAUD, ["PL0", 13] ;Stop the sequence now
    enable TIMERINT
    GPStart = 0
  ENDIF
return

;--------------------------------------------------------------------
;[GPSeqNumSteps] returns the number of steps a specific sequence has 
;		as defined in the variable GPSEQ has or -1 if the sequence is not defined
GPSeqNumSteps:
  serout cSSC_OUT, cSSC_BAUD, ["EER -", dec GPSEQ*2, ";2",13];Read adr to current seq
  serin cSSC_IN, cSSC_BAUD, 20000, _TO_GPSNS, [str GPSeqStart\2] ;read in 2 byte header - bugbug reuse a local variable
				
  IF (GPSeqStart = 0)  or (GPSeqStart = 0xffff)	THEN
    return -1;
  ENDIF

  serout cSSC_OUT, cSSC_BAUD, ["EER -", dec (GPSeqStart.highbyte*256+GPSeqStart.lowbyte)+2, ";1",13] 
  serin cSSC_IN, cSSC_BAUD, 20000,_TO_GPSNS, [GPCntSteps];
  return GPCntSteps

_TO_GPSNS:		 	
  return -1   ; SSC-32 did not respond, so assume not supported...


	
	
;--------------------------------------------------------------------
;[SERVO DRIVER] Updates the positions of the servos
;Binary version
;**********************************************************************
;Extended binary commands.
;
;Commands 0x80-0x9F are group move servo number 0-31.  The next 2 bytes
;must be pulse width for the servo:
;    0x80+servoNum, pwHigh, pwLow
;
;Command 0xA0 is the group move servo speed, which must immediately
;follow the pulse width:
;    0xA0, spdHigh, spdLow
;
;Command 0xA1 is the group move time, which must follow all of the
;pulse widths and speeds:
;    0xA1, timeHigh, timeLow
;
;Command 0xA2 is the stop all command, 1 byte
;    0xA2
;**********************************************************************
UpdateServoDriver:

  disable TIMERINT 		;disable timer interrupt
  for LegIndex = 0 to 5
    ; Calulate 
	GOSUB GetPWMValues [LegIndex]
	
    serout cSSC_OUT, cSSC_BAUD, [0x80+cCoxaPin(LegIndex) ,CoxaPWM.highbyte,  CoxaPWM.lowbyte,|
    	                         0x80+cFemurPin(LegIndex),FemurPWM.highbyte, FemurPWM.lowbyte,|
    	                         0x80+cTibiaPin(LegIndex),TibiaPWM.highbyte, TibiaPWM.lowbyte]
  next
  
  ;Send speed
  serout cSSC_OUT, cSSC_BAUD, [0xA1,SSCTime.highbyte,SSCTime.lowbyte]
  enable TIMERINT

  PrevSSCTime = SSCTime
return
;--------------------------------------------------------------------
;[FREE SERVOS] Frees all the servos (binary)
FreeServos
    disable TIMERINT
	for LegIndex = 0 to 31
      serout cSSC_OUT, cSSC_BAUD, [0x80+LegIndex, 0, 0]
    next
    serout cSSC_OUT, cSSC_BAUD, [0xA1, 0, 200]
    enable TIMERINT
return
;--------------------------------------------------------------------
;[COMMIT SERVO POSITIONS]
CommitServoDriver:
  ;Send <CR>
  disable TIMERINT
  serout cSSC_OUT, cSSC_BAUD, [13]	
  enable TIMERINT
return
;--------------------------------------------------------------------
	